#! /bin/bash
####################################################################
# benchmarks Mergsort speedup
# arguments are:
#  -o output filename
#  -w ws scheduler
#  -p pdf scheduler
#  -i input size, runsize = inputsize/runs
#  -e min run num runs
#  -r max number of runs
#  -t maximun number of threads
#  -a number of repetitions to average over
#  -j number of jobs used for merging
###################################################################
MergeSort="./Mergesort"
seqSort="../sequential_sort"
logname="mergesort_fanout_data.csv"

ws=false
pdf=false
inputsize=65536
maxRuns=64
minRuns=2
threads=4
reps=3
log=""
while getopts "wpli:r:e:t:a:o:" opt; do 
    case $opt in
	w)
	    ws=true
	    ;;
	p)
	    pdf=true
       	    ;;
	i)
	    inputsize=$OPTARG
	    ;;
	r)
	    maxRuns=$OPTARG
	    ;;
	e)
	    minRuns=$OPTARG
	    ;;
	t)
	    threads=$OPTARG
	    ;;
	a)
	    reps=$OPTARG
	    ;;
	o) 
	    logname=$OPTARG
	    ;;
	l)
	    log="--log"
	    ;;
    esac
done 

#clear the file
touch $logname
cat /dev/null > $logname

if $pdf
then
    heading="#scheduler, threads, runsize, numruns, tasks, repetitions, reps*(sorttime, mergetime)"
    echo $heading >> $logname
    runs=$minRuns
    until [ $runs -gt $maxRuns ]
    do
	runsize=$(( $inputsize / $runs ))
	rep=0
	line="PDF,${threads},${runsize},${runs},${threads},${reps}"
	while [ $rep -lt $reps ]
	do 
	    out="$( $MergeSort --PDF --threads $threads --runsize $runsize --runs $runs --tasks $threads $log )"
	    echo "running $MergeSort --PDF --threads $threads --runsize $runsize --runs $runs --tasks $threads $log (run sort, merge)= $out " >&2
	    line="${line},$out"
	    rep=$(($rep +1))
	done
	echo $line >> $logname
	let runs=$runs+$runs
    done
fi
if $ws
then
    runs=$minRuns
    until [ $runs -gt $maxRuns ]
    do
	runsize=$(( $inputsize / $runs ))
	rep=0
	line="WS,${threads},${runsize},${runs},${threads},${reps}"
	while [ $rep -lt $reps ]
	do 
	    out="$( $MergeSort --WS --threads $threads --runsize $runsize --runs $runs --tasks $threads $log )"
	    echo "running $MergeSort --WS --threads $threads --runsize $runsize --runs $runs --tasks $threads $log (run sort, merge) = $out " >&2
	    line="${line},$out"
	    rep=$(($rep +1))
	done
	echo $line >> $logname
	let runs=$runs+$runs
    done
fi
#run sequnetial
rep=0
line="SEQ,1,${inputsize},1,1,${reps}"
while [ $rep -lt $reps ]
do
    out="$($seqSort --input $inputsize $log)"
    line="${line},$out"
    echo "$seqSort --input $inputsize $log = $out" >&2
    rep=$(($rep +1))
done
echo $line >> $logname