#! /bin/bash

####################################################################
# benchmarks Quicksort grainularity
# arguments are:
#  -o output filename
#  -w ws scheduler
#  -p pdf scheduler
#  -i inputsize
#  -g min grainsize 
#  -h max grainsize
#  -t number of threads
#  -a number of repetitions to average over
#  -j number of tasks
#  a fourth of the grainsize is used for blocksize
###################################################################
QuickSort="./Quicksort"
seqSort="../sequential_sort"
logname="quicksort_granularity_data.csv"

ws=false
pdf=false
inputsize=65536
threads=2
reps=3
minGrain=32
maxGrain=4096
tasks=2
log=""
while getopts "wpli:g:h:t:a:j:o:" opt; do 
    case $opt in
	w)
	    ws=true
	    ;;
	p)
	    pdf=true
	    ;;
	l)
	    log="--log"
	    ;;
	i)
	    inputsize=$OPTARG
	    ;;
	g)
	    minGrain=$OPTARG
	    ;;
	h)
	    maxGrain=$OPTARG
	    ;;
	t)
	    threads=$OPTARG
	    ;;
	a)
	    reps=$OPTARG
	    ;;
	j)
	    tasks=$OPTARG
	    ;;
	o)
	    logname=$OPTARG
	    ;;
    esac
done 

#clear the file
touch $logname
cat /dev/null > $logname

heading="#scheduler, threads, inputsize, grainsize, numtasks, blocksize, repetitions, reps*(sorttime)"
echo $heading >> $logname

if $pdf
then
    grain=$minGrain
    until [ $grain -gt $maxGrain ]
    do
	blocksize=$(( $grain / 4 ))
	rep=0
	line="PDF,${threads},${inputsize},${grain},${tasks},${blocksize},${reps}"
	while [ $rep -lt $reps ]
	do 
	    out="$( $QuickSort --PDF --threads $threads --grain $grain --blocksize $blocksize --tasks $tasks --input $inputsize $log )"
	    echo "running $QuickSort --PDF --threads $threads --grain $grain --blocksize $blocksize --tasks $tasks --input $inputsize $log = $out " >&2
	    line="${line},$out"
	    rep=$(($rep +1))
	done
	echo $line >> $logname
	let grain=$grain+$grain
    done
fi
if $ws
then
    grain=$minGrain
    until [ $grain -gt $maxGrain ]
    do
	blocksize=$(( $grain / 4 ))
	rep=0
	line="WS,${threads},${inputsize},${grain},${tasks},${blocksize},${reps}"
	while [ $rep -lt $reps ]
	do 
	    out="$( $QuickSort --WS --threads $threads --grain $grain --blocksize $blocksize --tasks $tasks --input $inputsize $log )"
	     echo "running $QuickSort --WS --threads $threads --grain $grain --blocksize $blocksize --tasks $tasks --input $inputsize $log = $out " >&2
	    line="${line},$out"
	    rep=$(($rep +1))
	done
	echo $line >> $logname
	let grain=$grain+$grain
    done
fi
#run sequnetial
rep=0
line="SEQ,1,${inputsize},NA,NA,NA,${reps}"
while [ $rep -lt $reps ]
do
    out="$( $seqSort --input $inputsize $log )"
    line="${line},$out"
    echo "$seqSort --input $inputsize $log = $out" >&2
    rep=$(($rep +1))
done
echo $line >> $logname