#! /bin/bash

####################################################################
# benchmarks Quicksort speedup
# arguments are:
#  -o output filename
#  -w ws scheduler
#  -p pdf scheduler
#  -i inputsize
#  -g grainsize
#  -t maximun number of threads 1,2,4...max is used
#  -a number of repetitions to average over
#  -b number of tasks
#  -s number of elements per block
###################################################################
QuickSort="./Quicksort"
seqSort="../sequential_sort"
logname="quicksort_speedup_data.csv"

ws=false
pdf=false
inputsize=65536
maxThreads=4
numTasks=2
reps=3
grain=2048
numblocks=4
blocksize=128
log=""
while getopts "wpli:g:t:a:b:s:o:" opt; do 
    case $opt in
	w)
	    ws=true
	    ;;
	p)
	    pdf=true
	    ;;
	i)
	    inputsize=$OPTARG
	    ;;
	g)
	    grain=$OPTARG
	    ;;
	t)
	    maxThreads=$OPTARG
	    ;;
	a)
	    reps=$OPTARG
	    ;;
	b)
	    numTasks=$OPTARG
	    ;;
	s)
	    blocksize=$OPTARG
	    ;;
	o)
	    logname=$OPTARG
	    ;;
	l)
	    log="--log"
	    ;;
    esac
done 

#clear the file
touch $logname
cat /dev/null > $logname
heading="#scheduler, threads, inputsize, grainsize, numtasks, blocksize, repetitions, reps*(sorttime)"
echo $heading >> $logname

if $pdf
then
    thread=1
    until [ $thread -gt $maxThreads ]
    do
	rep=0
	line="PDF,${thread},${inputsize},${grain},${numTasks},${blocksize},${reps}"
	while [ $rep -lt $reps ]
	do 
	    out="$( $QuickSort --PDF --threads $thread --grain $grain --blocksize $blocksize --tasks $numTasks --input $inputsize $log )"
	    echo " $QuickSort --PDF --threads $thread --grain $grain --blocksize $blocksize --tasks $numTasks --input $inputsize $log = $out " >&2
	    line="${line},$out"
	    rep=$(($rep +1))
	done
	echo $line >> $logname
	let thread=$thread+$thread
    done
fi
if $ws
then
    thread=1
    until [ $thread -gt $maxThreads ]
    do
	rep=0
	line="WS,${thread},${inputsize},${grain},${numTasks},${blocksize},${reps}"
	while [ $rep -lt $reps ]
	do 
	    out="$( $QuickSort --WS --threads $thread --grain $grain --blocksize $blocksize --tasks $numTasks --input $inputsize $log )"
	    echo " $QuickSort --WS --threads $thread --grain $grain --blocksize $blocksize --tasks $numTasks --input $inputsize $log = $out " >&2
	    line="${line},$out"
	    rep=$(($rep +1))
	done
	echo $line >> $logname
	let thread=$thread+$thread
    done
fi
#run sequnetial
rep=0
line="SEQ,1,${inputsize},${grain},${numTasks},${blocksize},${reps}"
while [ $rep -lt $reps ]
do
    out="$( $seqSort --input $inputsize $log )"
    line="${line},$out"
    echo "$seqSort --input $inputsize $log = $out " >&2
    rep=$(($rep +1))
done
echo $line >> $logname