#include <cstdlib>
#include <time.h>

#include <boost/program_options.hpp>

#include <iostream>
#include <algorithm>
#include <string>
#include <math.h>

#include "MCL/array.h"

inline bool Cmp(size_t f, size_t s){
  return f<s;
}

inline bool LogCmp(size_t f, size_t s){
  return log(f)<log(s);
}

int main(int argc, char* argv[]) {
  namespace po = boost::program_options;
  // Declare the supported options.
  po::options_description desc("Allowed options");
  desc.add_options()
    ("help", "Produce help message")
    ("input", po::value<size_t>(), "Number of elements sorted")
    ("test", "Test that the result are sorted")
    ("log", "Logarithm comparison is used")
    ;
  po::variables_map args;
  po::store(po::parse_command_line(argc, argv, desc), args);
  po::notify(args);  
  if (args.count("help")) {
    std::cout << desc << "\n";
    return 1;
  }
  size_t inputSize = 65536;
  if(args.count("input"))
    inputSize = args["input"].as<size_t>();
  bool test = false;
  if(args.count("test"))
    test = true;
  bool log = false;
  if(args.count("log"))
    log = true;

  struct timespec start, end;
    
  mcl::BasicArray<size_t> input(inputSize);
  for(size_t i=0;i<input.size();++i){
    input[i] = i;
  }
  srand ( time(NULL) );
  std::random_shuffle(input.begin(),input.end());
  double sortTime;

  if(log) {
    clock_gettime(CLOCK_MONOTONIC, &start);
    std::sort(input.begin(),input.end(),LogCmp);
    clock_gettime(CLOCK_MONOTONIC, &end);
  } else {
    clock_gettime(CLOCK_MONOTONIC, &start);
    std::sort(input.begin(),input.end(),Cmp);
    clock_gettime(CLOCK_MONOTONIC, &end);
  }
  sortTime = (end.tv_sec - start.tv_sec);
  sortTime += (end.tv_nsec - start.tv_nsec) / 1000000000.0;
  std::cout<<sortTime;

  if(test){
    for(size_t i=0;i<input.size();++i){
      if(input[i] != i){
	std::cout<<" Failed"<<std::endl;
	return -1;
      }
    }
    std::cout<<" Passed"<<std::endl;
  }
  return 0;
}
