#ifndef MCL_BLOCKED_VIEW
#define MCL_BLOCKED_VIEW

/**
 * TODO:
 *      Redeclare iterator to use a baseview::iterator and move it forward with operator+=
 *       this allows a const iterator to be declared
 **/

namespace mcl {
 
  template<class BaseView>
  class BlockedIterator; 

  template<class BaseView>
  class BlockedView {
  public:
    typedef SubView<BaseView> value_type;
    typedef typename BaseView::key_type key_type;
    typedef typename BaseView::size_type size_type;
    typedef typename BaseView::index_type index_type;
    typedef SubView<BaseView> data_type;
    typedef SubView<BaseView>* pointer;
    typedef const SubView<BaseView>* const_pointer;    
    typedef SubView<BaseView> reference;
    typedef const SubView<BaseView> const_reference;
    typedef BlockedIterator<BaseView> iterator;
    typedef BlockedIterator<BaseView> const_iterator;

    inline explicit BlockedView(BaseView* base, size_t blockSize)
      : base_(base), blockSize_(blockSize) {}
    
    inline data_type operator[](const index_type& n) {
      size_type index(GetBlock(n));
      if(n == size()) {
	std::cout<<"returning end"<<std::endl;
	return data_type(base_,index,base_->size());
      }
      return data_type(base_,index,index+blockSize_);
    }
    inline iterator begin() {
      return iterator(this,0);
    }
    inline iterator end() {
      return iterator(this, size());
    }
    inline size_type size(){
      return ceil(static_cast<double>(base_->size())/blockSize_); 
    }

  protected:
    BaseView* base_;
    size_type blockSize_;
  private:
    //given a block number returns the index of the first element in the block
    inline index_type GetBlock(const index_type& n) {
      index_type ans = n*blockSize_;
    }
  };
  
  //SubView specialization
  template<class BaseView>
  class BlockedView<SubView<BaseView> >{
  public:
    typedef SubView<BaseView> value_type;
    typedef typename BaseView::key_type key_type;
    typedef typename BaseView::size_type size_type;
    typedef typename BaseView::index_type index_type;
    typedef SubView<BaseView> data_type;
    typedef SubView<BaseView>* pointer;
    typedef const SubView<BaseView>* const_pointer;    
    typedef SubView<BaseView> reference;
    typedef const SubView<BaseView> const_reference;
    typedef BlockedIterator<SubView<BaseView> > iterator;
    //const iterator is a weird thing here...
    typedef BlockedIterator<SubView<BaseView> > const_iterator;

    inline BlockedView(SubView<BaseView>* base, size_t blockSize)
      : base_(base,0,base->size()), blockSize_(blockSize), numBlocks_(ceil(static_cast<double>(base_.size())/blockSize_)) {}
   
    inline data_type operator[](const index_type& n) {
      size_type index(GetBlock(n));
      if (n == numBlocks_)
	throw std::range_error("index too large");
      if (n == numBlocks_-1) {
	return data_type(base_,index,base_.size());
      } else {
	return data_type(base_,index,index+blockSize_);
      }
    }
    inline iterator begin() {
      return iterator(this,0);
    }
    inline iterator end() {
      return iterator(this, size());
    }
    inline size_type size(){
      //std::cout<<"BlockedView<SubView>::size() base:"<<base_<<std::endl;
      return numBlocks_;
    }
    friend std::ostream& operator<<(std::ostream& out, const BlockedView<SubView<BaseView> >& bv) {
      bv.Print(out);
      return out;
    }
  protected:
    SubView<BaseView> base_;
    size_type blockSize_;
    //number of blocks including past the end
    size_type numBlocks_;
  private:
    //given a block number returns the index of the first element in the block
    inline index_type GetBlock(const index_type& n) {
      // if(n>=numBlocks_)
      // 	throw std::range_error("index too large");
      // if(n<0)
      // 	throw std::range_error("index is negative");
      index_type ans = n*blockSize_;
      return ans;
    }
    void Print(std::ostream& out) const{
      out<<"BlockedView(base:"<<base_<<" blockSize: "<<blockSize_<<")";
    }
  };

  template<class BaseView>
  class BlockedIterator {
  public:
    typedef typename BlockedView<BaseView>::value_type value_type;
    typedef typename BlockedView<BaseView>::data_type reference;
    typedef typename BaseView::iterator::difference_type difference_type;
    typedef typename BlockedView<BaseView>::pointer pointer;
    
    inline BlockedIterator() : base_(), pos_() {}
    
    inline BlockedIterator(BlockedView<BaseView>* b, typename BlockedView<BaseView>::index_type p)
      : base_(b), pos_(p) {}
   
    inline BlockedIterator(const BlockedIterator<BaseView>& rhs)
      : base_(rhs.base_), pos_(rhs.pos_) {}

    inline bool operator==(const BlockedIterator<BaseView>& rhs) const {
      return (base_ == rhs.base_ && pos_ == rhs.pos_);
    }
    
    inline bool operator!=(const BlockedIterator<BaseView>& rhs)  const {
      return !operator==(rhs);
    }
    
    inline bool operator<(const BlockedIterator<BaseView>& rhs) const {
      return (base_ == rhs.base_ && pos_ < rhs.pos_);
    }

    inline bool operator>(const BlockedIterator<BaseView>& rhs) const {
      return (base_ == rhs.base_ && pos_ > rhs.pos_);
    }
    
    inline bool operator<=(const BlockedIterator<BaseView>& rhs) const {
      return (operator<(rhs) || operator==(rhs));
    }

    inline bool operator>=(const BlockedIterator<BaseView>& rhs) const {
      return (operator>(rhs) || operator==(rhs));
    }

    inline reference operator*() {
      if(base_ == 0)
	return reference();
      return (*base_)[pos_];
    }
    inline reference operator->() {
      return (*base_)[pos_];
    }
    inline BlockedIterator<BaseView>& operator--(){
      --pos_;
      return *this;
    }
    inline BlockedIterator<BaseView>& operator++(){
      ++pos_;
      return *this;
    }
    inline BlockedIterator<BaseView> operator--(int){
      BlockedIterator<BaseView> ans(*this);
      --pos_;
      return ans;
    }
    inline BlockedIterator<BaseView> operator++(int){
      BlockedIterator<BaseView> ans(*this);
      ++pos_;
      return ans;
    }
    inline difference_type operator-(const BlockedIterator<BaseView>& rhs) const {
      return pos_-rhs-pos_;
    }
    inline BlockedIterator<BaseView>& operator+=(const difference_type& n){
      pos_ += n;
      return *this;
    }
    inline BlockedIterator<BaseView>& operator-=(const difference_type& n){
      pos_ -= n;
      return *this;
    }
    inline BlockedIterator<BaseView> operator+(const difference_type& n) const {
      BlockedIterator<BaseView> ans(*this);
      ans.pos_ += n;
      return ans;
    }
    inline BlockedIterator<BaseView> operator-(const difference_type& n) const {
      BlockedIterator<BaseView> ans(*this);
      ans.pos_ -= n;
      return ans;
    }
  protected:
    BlockedView<BaseView>* base_;
    typename BlockedView<BaseView>::index_type pos_;
  };

}

#endif

